import { Injectable } from "@angular/core";
import { Router } from "@angular/router";
import { Actions, createEffect, ofType } from "@ngrx/effects"
import { Store } from "@ngrx/store";
import { EMPTY, of } from "rxjs";
import { catchError, map, mergeMap, withLatestFrom } from "rxjs/operators";
import { AppState } from "src/app/app.reducers";
import AuthenticatedUser from "src/app/common/models/authenticated-user";
import Usuario from "src/app/common/models/usuario";
import { AuthenticationService } from "src/app/common/services/authentication.service";
import { UsuariosService } from "src/app/common/services/usuarios.service";
import * as desktopActions from "../actions";

@Injectable()
export class DesktopEffects {

    authenticateUser = createEffect(() =>
        this.action$.pipe(
            ofType(desktopActions.authenticateUser),
            mergeMap((data:{user_name:string}) =>
                this.authenticationSvc.authenticate(data.user_name).pipe(
                    map((authenticatedUser: AuthenticatedUser) => 
                        desktopActions.authenticateUserOk({user:authenticatedUser})
                    ),
                    catchError((error) => of(desktopActions.authenticateUserError({error:error})))
                )
            )
        )
    );

    authenticateUserOk = createEffect(() =>
        this.action$.pipe(
            ofType(desktopActions.authenticateUserOk),
            mergeMap(({user}) => {
                this.authenticationSvc.currentUser = user;
                this.router.navigate(['']);
                return EMPTY;
            })
        ),
        {dispatch:false}
    );

    authenticateUserError = createEffect(() =>
        this.action$.pipe(
            ofType(desktopActions.authenticateUserError),
            mergeMap(() => {
                this.authenticationSvc.currentUser = null;
                this.router.navigate(['no_auth']);
                return EMPTY;
            })
        ),
        {dispatch: false}
    );

    logoutUser = createEffect(() =>
        this.action$.pipe(
            ofType(desktopActions.logoutUser),
            mergeMap(() => {
                this.authenticationSvc.currentUser = null;
                this.router.navigate(['logout']);
                return EMPTY;
            })
        ),
        {dispatch: false}
    );

    registerApplication = createEffect(() =>
        this.action$.pipe(
            ofType(desktopActions.registerApplication),
            withLatestFrom(this.store.select("desktop")),
            mergeMap(([{app}, desktopState]) => {
                if (!desktopState.registeredApps.some(existing_app => existing_app.name === app.name)) {
                    this.router.resetConfig([
                        ...this.router.config,
                        {
                            path: app.path,
                            children: app.routes
                        }
                    ])
                    return of(desktopActions.registerApplicationOk({app}))
                } else {
                    return EMPTY;
                }
            })
        )
    );

    openApplication = createEffect(() =>
        this.action$.pipe(
            ofType(desktopActions.openApplication),
            mergeMap(({app}) => {
                this.router.navigate([app.path]);
                return of(desktopActions.openApplicationOk({app}))
            })
        )
    );

    openDesktop = createEffect(() =>
        this.action$.pipe(
            ofType(desktopActions.openDesktop),
            mergeMap(() => {
                this.router.navigate(['/']);
                return of(desktopActions.openDesktopOk())
            })
        )
    );

    loadUsers = createEffect(() =>
        this.action$.pipe(
            ofType(desktopActions.loadUsers),
            mergeMap(() =>
                this.usuariosSvc.getAll().pipe(
                    map((usuarios:Usuario[]) =>
                        desktopActions.loadUsersOk({users:usuarios})
                    ),
                    catchError((error) => of(desktopActions.loadUsersError({error:error})))
                )
            )
        )
    );

    constructor(
        private action$: Actions,
        private router: Router,
        private store: Store<AppState>,
        private usuariosSvc: UsuariosService,
        private authenticationSvc: AuthenticationService
    ) { }
}